# -*- coding: utf-8 -*-

import os
import json
import pickle
import argparse

from sklearn.linear_model import LogisticRegression
from sklearn.tree import DecisionTreeClassifier
from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.svm import SVC
from sklearn.pipeline import make_pipeline
from sklearn.preprocessing import StandardScaler


models = {
    'LR': LogisticRegression(),
    'LR_SS': make_pipeline(StandardScaler(), LogisticRegression()),
    'MLP': MLPClassifier(),
    'KNN': KNeighborsClassifier(),
    'SVM': SVC(),
    'DT': DecisionTreeClassifier(),
    'RF': RandomForestClassifier(),
    'Ada': AdaBoostClassifier(),
}


def main(args):
    with open(os.path.join(args.data_path, 'all_train.json'), 'r') as f:
        train_data = json.load(f)

    x_train, y_train = [], []
    for x, y in train_data:
        x_train.append(x)
        y_train.append(y)

    model = models[args.model]
    print('开始训练模型 ', args.model)
    model.fit(x_train, y_train)
    print('模型训练完成')

    model_save_path = os.path.join(args.save_path, args.model + '.pkl')
    with open(model_save_path, 'wb') as f:
        pickle.dump(model, f)
        print('模型已保存至 ', model_save_path)

    if args.do_test:
        with open('data/all_test.json', 'r') as f:
            test_data = json.load(f)

        x_test, y_test = [], []
        for x, y in test_data:
            x_test.append(x)
            y_test.append(y)

        print('{}\t{}'.format(args.model, model.score(x_test, y_test)))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='壹点灵 咨询师推荐 精排模型训练')
    parser.add_argument('-m', '--model', type=str, default='LR', choices=list(models.keys()), help='模型类型')
    parser.add_argument('--data_path', type=str, default='./data', help='训练数据存放目录')
    parser.add_argument('--save_path', type=str, default='./model', help='训练好的模型存放目录')
    parser.add_argument('--do_test', default=False, action='store_true', help='训练完成后执行测试程序')
    args = parser.parse_args()
    main(args)